package ict11.elgamal;

import java.math.BigInteger;
import java.util.Random;

public class ElGamal {

	public static EncryptedMessage encrypt(BigInteger [] message, ElGamalPublicKey key) {
		Random rnd = new Random();
		BigInteger [] k = new BigInteger[message.length];
			
		for (int i=0; i< k.length; i++) 
			k[i] = new BigInteger(key.p.bitCount(), rnd);
		
		BigInteger [] delta = new BigInteger[message.length];
		BigInteger [] gamma = new BigInteger[message.length];

		for (int i=0; i<message.length; i++) {
			gamma[i] = key.alpha.modPow(k[i],key.p);
			BigInteger alphaAK = key.alphaA.modPow(k[i],key.p);
			delta[i] = message[i].multiply(alphaAK).mod(key.p);
		}
		return new EncryptedMessage(gamma,delta);
	}

	public static BigInteger [] decrypt(EncryptedMessage c, ElGamalPrivateKey key) {
		BigInteger p = key.getP();

		
		//decrypt using m = gamma^(-a) * delta (mod p)
		BigInteger [] message = new BigInteger[c.delta.length];
		for (int i=0; i<c.delta.length; i++)  {
			//gamma^(-a) = gamma^(p-1-a)
			BigInteger gamma_A = c.gamma[i].modPow(p.subtract(BigInteger.ONE).subtract(key.a),p);
			message[i] = c.delta[i].multiply(gamma_A).mod(p);
		}
		return message;
	}

	//generate random key, random message. Encrypt, decrypt, check.
	public static void main(String [] args) {
		int order = 0;
		try {
			order = Integer.parseInt(args[0]);
		}
		catch (Exception e) {
			System.out.println("Specify a order for the key. E.g. ElGamal 25");
			return;
		}
		System.out.println("Generating key...");
		ElGamalPrivateKey priv = ElGamalKeyGenerator.generate(order);
		System.out.println(priv.publicKey+"\r\n"+priv);

		Random rnd = new Random();
		BigInteger [] message = new BigInteger[10];
		for (int i=0; i<message.length; i++) 
			message[i] = new BigInteger(order-1,rnd);

		System.out.println("Encrypting...");
		EncryptedMessage cipher = encrypt(message,priv.publicKey);

		System.out.println("Decrypting...");
		BigInteger [] renewed = decrypt(cipher, priv);

		System.out.println("Verifying...");
		for (int i=0; i<message.length; i++) {
			if (message[i].compareTo(renewed[i]) != 0) 
				System.out.println("We have an error");
			else
				System.out.println("Match");
/*
			System.out.println("Message:\t"+message[i].toString());
			System.out.println("Renewed:\t"+renewed[i].toString());
			System.out.println("Cipher: \t"+cipher.delta[i].toString());
*/
		}
	}
	
}
