package ict11.elgamal;

import java.math.BigInteger;
import java.util.StringTokenizer;

public class ElGamalPrivateKey {

	public ElGamalPublicKey publicKey;
	public BigInteger a;

	public ElGamalPrivateKey(BigInteger p, BigInteger alpha, BigInteger a) {
		BigInteger alphaA = alpha.modPow(a,p);
		publicKey = new ElGamalPublicKey(p,alpha,alphaA);
		this.a=a;
	}

	public ElGamalPrivateKey(String key) throws Exception {
		StringTokenizer st = new StringTokenizer(key,"(,)");
		String preamble = st.nextToken();
		BigInteger p = new BigInteger(st.nextToken());
		BigInteger alpha = new BigInteger(st.nextToken());
		a = new BigInteger(st.nextToken());
		publicKey = new ElGamalPublicKey(p,alpha,alpha.modPow(a,p));
	}

	public BigInteger getP() {return publicKey.p; }

	public String toString() {
		return "ElGamalPrivate:("+publicKey.p.toString()+","+publicKey.alpha.toString()+","+a+")";
	}

	public String toString(int i) {
		return publicKey.toString()+"\r\nPrivate Key:\r\na="+a.toString();
	}

	/* main method tests the string parsing ability*/
	public static void main(String []args) {
		if (args.length==0) {
			System.out.println("Specify a private key to parse");
			return;
		}
		try {
			ElGamalPrivateKey p = new ElGamalPrivateKey(args[0]);
			System.out.println("Correctly Parsed:\r\n"+p.toString()+"\r\n"+p.publicKey.toString());
		}
		catch (Exception e) {
			System.out.println("Could not parse\r\n"+e.toString());
		}
	}
}
